/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/ce/Engine.h>
#include <ti/sdo/ce/CERuntime.h>

#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Sound.h>
#include <ti/sdo/dmai/Buffer.h>
#include <ti/sdo/dmai/ce/Aenc.h>

#include <ti/sdo/codecs/aacenc/iaacenc.h>

#include "appMain.h"

#define SAMPLE_RATE     44100
#define BIT_RATE        256000

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    Buffer_Attrs            bAttrs          = Buffer_Attrs_DEFAULT;
    Sound_Attrs             sAttrs          = Sound_Attrs_STEREO_DEFAULT;
    AUDENC_Params           params          = Aenc_Params_DEFAULT;
    AUDENC_DynamicParams    dynamicParams   = Aenc_DynamicParams_DEFAULT;
    Sound_Handle            hSound          = NULL;
    FILE                   *outFile         = NULL;
    Aenc_Handle             hAe             = NULL;
    Engine_Handle           hEngine         = NULL;
    Buffer_Handle           hInBuf          = NULL;
    Buffer_Handle           hOutBuf         = NULL;
    Int                     frameNbr        = 0;

    outFile = fopen(args->outFile, "wb");

    if (outFile == NULL) {
        printf("Failed to open output file %s\n", args->outFile);
        goto cleanup;
    }

    /* Initialize the codec engine run time */
    CERuntime_init();

    /* Initialize DMAI */
    Dmai_init();
    
    /* Open the codec engine */
    hEngine = Engine_open(args->engineName, NULL, NULL);

    if (hEngine == NULL) {
        printf("Failed to open codec engine %s\n", args->engineName);
        goto cleanup;
    }

    /* Create the audio encoder */
    params.maxBitrate = BIT_RATE;
    params.maxSampleRate = SAMPLE_RATE;

    hAe = Aenc_create(hEngine, args->codecName, &params, &dynamicParams);

    if (hAe == NULL) {
        printf("Failed to create audio encoder %s\n", args->codecName);
        goto cleanup;
    }

    /* Create the sound device driver interface */
    sAttrs.sampleRate = SAMPLE_RATE;
    hSound = Sound_create(&sAttrs);

    if (hSound == NULL) {
        printf("Failed to create audio device\n");
        goto cleanup;
    }

    hInBuf = Buffer_create(Aenc_getInBufSize(hAe), &bAttrs);
    hOutBuf = Buffer_create(Aenc_getOutBufSize(hAe), &bAttrs);

    if (hInBuf == NULL || hOutBuf == NULL) {
        printf("Failed to get free contiguous buffers\n");
        goto cleanup;
    }

    while (frameNbr < 200) {
        if (Sound_read(hSound, hInBuf) < 0) {
            printf("Failed to read audio buffer\n");
            goto cleanup;
        }

        /* Encode the audio buffer */
        if (Aenc_process(hAe, hInBuf, hOutBuf) < 0) {
            printf("Failed to encode audio buffer\n");
            goto cleanup;
        }

        /* Write the encoded frame to the file system */
        if (Buffer_getNumBytesUsed(hOutBuf)) {
            if (fwrite(Buffer_getUserPtr(hOutBuf),
                       Buffer_getNumBytesUsed(hOutBuf), 1, outFile) != 1) {
                printf("Failed to write encoded audio data to disc\n");
                goto cleanup;
            }
        }
        else {
            printf("Warning: Encoder returned zero byte frame\n");
        }

        printf("frameNbr = %d\n", frameNbr++);
    }

cleanup:
    if (hOutBuf) {
        Buffer_delete(hOutBuf);
    }

    if (hInBuf) {
        Buffer_delete(hInBuf);
    }

    if (hAe) {
        Aenc_delete(hAe);
    }

    if (hSound) {
        Sound_delete(hSound);
    }

    if (hEngine) {
        Engine_close(hEngine);
    }

    if (outFile) {
        fclose(outFile);
    }

    return;
}
