/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <xdc/std.h>

#include <ti/sdo/dmai/Cpu.h>
#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Time.h>
#include <ti/sdo/dmai/BufferGfx.h>

#include <ti/sdo/dmai/Framecopy.h>

#include "appMain.h"

/******************************************************************************
 * appMain
 *****************************************************************************/
Int appMain (Args *argsp)
{
    Int                 ret =  Dmai_EOK, count = 0;
    Uint32              deltaTime;
    Int                 cpuLoad;
    Cpu_Handle          hCpu = NULL;
    Cpu_Attrs           cpuAttrs = Cpu_Attrs_DEFAULT;          
    Time_Attrs          tAttrs = Time_Attrs_DEFAULT;
    Time_Handle         hTime = NULL;
    Framecopy_Attrs     fcAttrs = Framecopy_Attrs_DEFAULT;
    Framecopy_Handle    hFc = NULL;    
    Buffer_Handle       inBuf, outBuf;
    BufferGfx_Attrs     gfxAttrs = BufferGfx_Attrs_DEFAULT;
    Uns                 size;

    /* Initialize DMAI */
    Dmai_init();

    if (argsp->benchmark) {
        hTime   = Time_create (&tAttrs);

        if (hTime == NULL) {
            fprintf (stderr, "Failed to create time object\n");
            ret = Dmai_EFAIL;
            goto cleanup;
        }
        
        hCpu = Cpu_create (&cpuAttrs);

        if (hCpu == NULL) {
            fprintf (stderr, "Failed to create cpu object\n");
            ret  = Dmai_EFAIL;
            goto cleanup;
        }
    }

    gfxAttrs.dim.width      = argsp->width;
    gfxAttrs.dim.height     = argsp->height;
    gfxAttrs.dim.lineLength = argsp->width * 2;
    gfxAttrs.colorSpace     = ColorSpace_UYVY;
    size                    = gfxAttrs.dim.lineLength * argsp->height;
 
    inBuf = Buffer_create (size, BufferGfx_getBufferAttrs(&gfxAttrs));

    if (inBuf == NULL) {
        fprintf (stderr, "Failed to allocate memory for input buffer.\n");
        ret = Dmai_EFAIL;
        goto cleanup;
    }
   
    outBuf = Buffer_create (size, BufferGfx_getBufferAttrs(&gfxAttrs));

    if (outBuf == NULL) {
        fprintf (stderr, "Failed to allocate memory for output buffer.\n");
        ret = Dmai_EFAIL;
        goto cleanup;
    }

    if (argsp->benchmark) 
        Time_reset (hTime);

    fcAttrs.accel = TRUE;        
    hFc = Framecopy_create(&fcAttrs);
    
    if (hFc == NULL) {
        fprintf (stderr, "Failed to open framecopy module.\n");
        ret = Dmai_EFAIL;
        goto cleanup;
    }

    if (Framecopy_config (hFc, inBuf, outBuf) < 0) {
        fprintf (stderr, "Failed to configure framecopy.\n");
        ret = Dmai_EFAIL;
        goto cleanup;
    }

    while (count != argsp->numFrames) {

        if (Framecopy_execute(hFc, inBuf, outBuf) < 0) {
            fprintf ( stderr, "failed to copy frame %d\n", count); 
            ret = Dmai_EFAIL;
            goto cleanup;
        }

        printf ("\rProcessing frame %d(%d)      ", ++count, argsp->numFrames);
        fflush(stdout);
    }
   
    printf ("\n"); 
   
    if (argsp->benchmark) {
        Time_total (hTime, &deltaTime);
        Cpu_getLoad (hCpu, &cpuLoad);
        printf ("Cpu load: %d\n", cpuLoad);
        printf ("Time: %lu us, %.5f s\n", 
                    deltaTime, (Float)deltaTime/1000000);
        printf ("fps: %.2f\n",
                (Float)argsp->numFrames/(Float)(deltaTime/1000000));
        printf("Throughput: %.2f MB/s\n", 
        ((count*Buffer_getSize(inBuf))/((Float)deltaTime/1000000))/(1024*1024));
    }
        
cleanup:

    if (inBuf)
        Buffer_delete (inBuf);

    if (outBuf)
        Buffer_delete (outBuf);

    if (hFc)        
        Framecopy_delete (hFc);

    if (hTime)
        Time_delete (hTime);

    if (hCpu) 
        Cpu_delete (hCpu);

    return ret;
}
 
