/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/ce/Engine.h>
#include <ti/sdo/ce/CERuntime.h>

#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Sound.h>
#include <ti/sdo/dmai/Loader.h>
#include <ti/sdo/dmai/BufTab.h>
#include <ti/sdo/dmai/Time.h>
#include <ti/sdo/dmai/ce/Sdec.h>

#include "appMain.h"

#define BUFSIZE 160
#define NUM_BUFS 2

static Char vbuffer[20480];

/******************************************************************************
 * writeFrame
******************************************************************************/
static Int writeFrame(Buffer_Handle hBuf, FILE *outFile)
{
    Int8 *ptr = Buffer_getUserPtr(hBuf);
    Int32 numBytes;
    
    numBytes = Buffer_getNumBytesUsed(hBuf);

    if (fwrite(ptr, numBytes, 1, outFile) != 1) {
        printf("Failed to write data to disk\n");
        return -1;
    }

    printf("Wrote frame size %d bytes to disk\n", (Int)numBytes);

    return 0;
}

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    Loader_Attrs            lAttrs      = Loader_Attrs_DEFAULT;
    Buffer_Attrs            bAttrs      = Buffer_Attrs_DEFAULT;
    SPHDEC_Params           params      = Sdec_Params_DEFAULT;
    SPHDEC_DynamicParams    dynParams   = Sdec_DynamicParams_DEFAULT;
    Sdec_Handle             hSd         = NULL;
    Loader_Handle           hLoader     = NULL;
    Engine_Handle           hEngine     = NULL;
    Buffer_Handle           hOutBuf     = NULL;
    Time_Handle             hTime       = NULL;
    Time_Attrs              tAttrs      = Time_Attrs_DEFAULT;
    Buffer_Handle           hInBuf;
    Int                     numFrame     = 0;
    UInt32                  time;
    FILE                    *outFile;
    
    if (args->benchmark) {
        hTime = Time_create(&tAttrs);

        if (hTime == NULL) {
            printf("Failed to create Time object\n");
            goto cleanup;
        }
    }

    /* Initialize the codec engine run time */
    CERuntime_init();

    /* Initialize DMAI */
    Dmai_init();
    
    /* Open output file */
    outFile = fopen(args->outFile, "wb");
    setvbuf(outFile, vbuffer, _IOFBF, sizeof(vbuffer));
    
    if (outFile == NULL) {
        printf("Failed to create output file %s\n", args->outFile);
        goto cleanup;
    }

    /* Open the codec engine */
    hEngine = Engine_open(args->engineName, NULL, NULL);

    if (hEngine == NULL) {
        printf("Failed to open codec engine %s\n", args->engineName);
        goto cleanup;
    }

    /* Create the XDM 0.9 based speech decoder */
    hSd = Sdec_create(hEngine, args->codecName, &params, &dynParams);

    if (hSd == NULL) {
        printf("Failed to create speech decoder\n");
        goto cleanup;
    }

    lAttrs.readSize = BUFSIZE;
    lAttrs.readBufSize = 1 * 1024 * 1024;

    /* Create the file loader */
    hLoader = Loader_create(args->inFile, &lAttrs);

    if (hLoader == NULL) {
        printf("Failed to create loader\n");
        goto cleanup;
    }

    hOutBuf = Buffer_create(BUFSIZE * 8, &bAttrs); // TODO ask codec

    /* Prime the file loader */
    Loader_prime(hLoader, &hInBuf);

    printf("Loader created and primed, entering main loop..\n");

    while (numFrame++ < args->numFrames) {
        if (args->benchmark) {
            if (Time_reset(hTime) < 0) {
                printf("Failed to reset timer\n");
                goto cleanup;
            }
        }

#ifdef CACHE_ENABLED
        /*  
         *  To meet xDAIS DMA Rule 7, we must writeback the cache into physical
         *  memory.  Also, per DMA Rule 7, we must invalidate the buffer's
         *  cache before providing it to any xDAIS algorithm.
         */
        Memory_cacheWbInv(Buffer_getUserPtr(hInBuf), Buffer_getSize(hInBuf));

        /* Per DMA Rule 7, our destination buffer cache lines must be cleaned */
        Memory_cacheInv(Buffer_getUserPtr(hOutBuf), Buffer_getSize(hOutBuf));
#endif
        /* Decode the video buffer */
        if (Sdec_process(hSd, hInBuf, hOutBuf) < 0) {
            printf("Failed to decode video buffer\n");
            goto cleanup;
        }

        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }

            printf("Frame %d - Decode: %uus \n", numFrame, (unsigned int)time);
        }
#ifdef CACHE_ENABLED
        /* Writeback the outBuf. */
        Memory_cacheWb(Buffer_getUserPtr(hOutBuf), Buffer_getSize(hOutBuf));
#endif
              
        if (numFrame >= args->startFrame) {
            printf("Frame %d: ", numFrame);                  
            if (writeFrame(hOutBuf, outFile) < 0) {
                goto cleanup;
            }
        }
            
        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }
            printf("Time - Writing frame: %uus ", (unsigned int)time);
        }

        /* Load a new frame from the file system */
        Loader_getFrame(hLoader, hInBuf);

        if (Buffer_getUserPtr(hInBuf) == NULL) {
            printf("Loader returned null, clip finished\n");
            break;
        }

        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }
            printf("Loader: %uus ", (unsigned int)time);
        }

        if (args->benchmark) {
            if (Time_total(hTime, &time) < 0) {
                printf("Failed to get timer total\n");
                goto cleanup;
            }
            printf("Total: %uus\n", (unsigned int)time);
        }
    }

cleanup:
    /* Clean up the application */
    if (hLoader) {
        Loader_delete(hLoader);
    }

    if (hSd) {
        Sdec_delete(hSd);
    }

    if (hOutBuf) {
        Buffer_delete(hOutBuf);
    }

    if (hEngine) {
        Engine_close(hEngine);
    }

    return;
}
