/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/ce/Engine.h>
#include <ti/sdo/ce/CERuntime.h>

#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Sound.h>
#include <ti/sdo/dmai/Buffer.h>
#include <ti/sdo/dmai/ce/Senc.h>

#include "appMain.h"

#define NUM_BUFS    2
#define INBUF_SIZE  320         // x2 compression rate and stereo samples
#define OUTBUF_SIZE 80


/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    SPHENC_Params           params      = Senc_Params_DEFAULT;
    SPHENC_DynamicParams    dynParams   = Senc_DynamicParams_DEFAULT;
    Buffer_Attrs            bAttrs      = Buffer_Attrs_DEFAULT;
    Sound_Attrs             sAttrs      = Sound_Attrs_MONO_DEFAULT;
    Sound_Handle            hSound      = NULL;
    FILE                   *outFile     = NULL;
    Senc_Handle             hSe         = NULL;
    Engine_Handle           hEngine     = NULL;
    Buffer_Handle           hInBuf      = NULL;
    Buffer_Handle           hOutBuf     = NULL;
    Int                     frameNbr    = 0;


    outFile = fopen(args->outFile, "wb");

    if (outFile == NULL) {
        printf("Failed to open output file %s\n", args->outFile);
        goto cleanup;
    }

    /* Initialize the codec engine run time */
    CERuntime_init();

    /* Initialize DMAI */
    Dmai_init();
    
    /* Open the codec engine */
    hEngine = Engine_open(args->engineName, NULL, NULL);

    if (hEngine == NULL) {
        printf("Failed to open codec engine %s\n", args->engineName);
        goto cleanup;
    }

    /* Create the speech encoder */
    hSe = Senc_create(hEngine, args->codecName, &params, &dynParams);

    if (hSe == NULL) {
        printf("Failed to create speech encoder %s\n", args->codecName);
        goto cleanup;
    }

    /* Create the sound device driver interface */
    sAttrs.mode = Sound_Mode_INPUT;
    hSound = Sound_create(&sAttrs);

    if (hSound == NULL) {
        printf("Failed to create audio device\n");
        goto cleanup;
    }

    hInBuf = Buffer_create(INBUF_SIZE, &bAttrs);
    hOutBuf = Buffer_create(OUTBUF_SIZE, &bAttrs);

    if (hInBuf == NULL || hOutBuf == NULL) {
        printf("Failed to get create contiguous buffers\n");
        goto cleanup;
    }

    while (frameNbr++ < 1000) {
        if (Sound_read(hSound, hInBuf) < 0) {
            printf("Failed to read audio buffer\n");
            goto cleanup;
        }

        /* Encode the speech buffer */
        if (Senc_process(hSe, hInBuf, hOutBuf) < 0) {
            printf("Failed to encode speech buffer\n");
            goto cleanup;
        }

        /* Write the encoded frame to the file system */
        if (fwrite(Buffer_getUserPtr(hOutBuf),
                   Buffer_getNumBytesUsed(hOutBuf), 1, outFile) != 1) {
            printf("Failed to write encoded speech data to disc\n");
            goto cleanup;
        }
    }

cleanup:
    if (hInBuf) {
        Buffer_delete(hInBuf);
    }

    if (hOutBuf) {
        Buffer_delete(hOutBuf);
    }

    if (hSe) {
        Senc_delete(hSe);
    }

    if (hSound) {
        Sound_delete(hSound);
    }

    if (hEngine) {
        Engine_close(hEngine);
    }

    if (outFile) {
        fclose(outFile);
    }

    return;
}
