/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>

#include <xdc/std.h>

#include <ti/sdo/dmai/Fifo.h>
#include <ti/sdo/dmai/Pause.h>
#include <ti/sdo/dmai/Rendezvous.h>

#include "appMain.h"

Fifo_Handle hFifo;
Pause_Handle hPause;    
Rendezvous_Handle hRv, hRvAll;


/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain()
{
    Fifo_Attrs fAttrs = Fifo_Attrs_DEFAULT;
    Int message[] = { 1, 2, 3 };
    Pause_Attrs pAttrs = Pause_Attrs_DEFAULT;
    Rendezvous_Attrs rzvAttrs = Rendezvous_Attrs_DEFAULT;
              
    printf("Synchronization Test started\n");
    
    /* Initialize DMAI */
    Dmai_init();
    
    /* Create objects */
    hPause = Pause_create(&pAttrs);
    if (hPause == NULL) {
        printf("Failed to create Pause object\n");
        return;
    }  

    hRv = Rendezvous_create(2, &rzvAttrs);
    if (hRv == NULL) {
        printf("Failed to create Rendezvous object\n");
        goto cleanup;
    }  

    hRvAll = Rendezvous_create(Rendezvous_INFINITE, &rzvAttrs);
    if (hRvAll == NULL) {
        printf("Failed to create Super Rendezvous object\n");
        goto cleanup;
    } 
        
    hFifo = Fifo_create(&fAttrs);
    if (hFifo == NULL) {
        printf("Failed to create Fifo object\n");
        goto cleanup;
    }  

    if (!createThread2()) {
        goto cleanup;
    }

    if (!createThread3()) {
        goto cleanup;
    }
        
    printf("Thread 1: At rendezvous point.\n");     
    Rendezvous_meet(hRv);
    printf("Thread 1: Rendezvous met. Turning on pause...\n");          
    Pause_on(hPause);
    printf("Thread 1: Pause On. Sending message...\n");
    if (Fifo_put(hFifo, message) != Dmai_EOK) {
        printf("Thread 1: Fifo_put failed.\n");
        goto cleanup;       
    }
    printf("Thread 1: Message put. Turning off pause...\n");
    Pause_off(hPause);
    printf("Thread 1: Pause off.\n");

    printf("Thread 1: At Super rendezvous point.\n");   
    Rendezvous_force(hRvAll);
    printf("Thread 1: Super Rendezvous forced.\n"); 

cleanup:
    if (hFifo) {
        Fifo_delete(hFifo);
    }
    
    if (hPause) {
        Pause_delete(hPause);
    }
    
    if (hRv) {
        Rendezvous_delete(hRv);
    }

    if (hRvAll) {
        Rendezvous_delete(hRvAll);
    }
    
    printf("Thread 1: end.\n");    
    return;
}

/******************************************************************************
 * thr2Main
 ******************************************************************************/
Void thr2Main()
{
    Int *msgPtr;

    printf("Thread 2: At rendezvous point.\n"); 
    Rendezvous_meet(hRv);
    printf("Thread 2: Rendezvous met. Waiting for message...\n");   
    if (Fifo_get(hFifo, &msgPtr) != Dmai_EOK) {
        printf("Thread 2: Fifo_get failed.\n");
        goto cleanup;
    }
    printf("Thread 2: Message received. Message is %d, %d, %d.\n", msgPtr[0],
        msgPtr[1], msgPtr[2]);
    printf("Thread 2: Going into pause.\n");
       
    Pause_test(hPause);

    printf("Thread 2: Resumed execution.\n");

    printf("Thread 2: At Super rendezvous point.\n");   
    Rendezvous_meet(hRvAll);
    printf("Thread 2: Super Rendezvous met.\n");    

cleanup:
   
    printf("Thread 2: end.\n");    
    
    return;
}

/******************************************************************************
 * thr3Main
 ******************************************************************************/
Void thr3Main()
{
    printf("Thread 3: At Super rendezvous point.\n");   
    Rendezvous_meet(hRvAll);
    printf("Thread 3: Super Rendezvous met.\n");    

    printf("Thread 3: Trying Super rendezvous again... should not block!\n");   
    Rendezvous_meet(hRvAll);
    printf("Thread 3: Super Rendezvous met.\n");
       
    printf("Thread 3: end.\n");    
    
    return;
} 
