/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>

#include <xdc/std.h>

#include "../appMain.h"

#define DEFAULT_ENGINE_NAME     "encode"

/*
 * Argument IDs for long options. They must not conflict with ASCII values,
 * so start them at 256.
 */
typedef enum
{
   ArgID_BENCHMARK = 256,
   ArgID_CODEC,
   ArgID_ENGINE,
   ArgID_HELP,
   ArgID_OUTPUT_FILE,
   ArgID_NUMFRAMES,
   ArgID_VIDEOINPUT,
   ArgID_BITRATE,
   ArgID_ACCEL,
   ArgID_CAPTURE_DALLOC,
   ArgID_XDM,
   ArgID_NUMARGS
} ArgID;

#define DEFAULT_ARGS \
    { Capture_Input_COMPOSITE, 30 * 30, Xdm1, -1, FALSE, FALSE, FALSE }
        
/******************************************************************************
 * usage
 ******************************************************************************/
static Void usage(void)
{
    fprintf(stderr, "Usage: video_capture_encode_<platform> [options]\n\n"
        "Options:\n"
        "   | --benchmark      Benchmark values are printed.\n"
        "-c | --codec          Name of codec to use\n"
        "-e | --engine         Codec engine containing specified codec\n"
        "   | --videoinput     Which video input to use (see below)\n"
        "-b | --bitrate        Bit rate to encode video at [variable]\n"
        "-n | --numframes      Number of frames to encode [Default: 1000].\n"
        "   | --accel          Use hardware acceleration if available for\n"
        "                      output color space conversion or frame copy.\n"
        "   | --capture_dalloc Have the capture driver allocate own buffers.\n"
        "   | --xdm            Which XDM version to use [1.0 default or 0.9].\n"
        "-h | --help           Print usage information (this message)\n"
        "-o | --output_file    Name of output file\n"
        "Video inputs available:\n"
        "\t1\tComposite [Default]\n"
        "\t2\tComponent\n"
        "\t3\tS-Video\n\n"
        "At a minimum the codec name and the file name *must* be given.\n\n");
}

/******************************************************************************
 * parseArgs
 ******************************************************************************/
static Void parseArgs(Int argc, Char *argv[], Args *argsp)
{  
    const char shortOptions[] = "c:e:b:n:ho:";
    const struct option longOptions[] = {
        {"benchmark",      no_argument,       NULL, ArgID_BENCHMARK   },
        {"codec",          required_argument, NULL, ArgID_CODEC       },
        {"engine",         required_argument, NULL, ArgID_ENGINE      },
        {"numframes",      required_argument, NULL, ArgID_NUMFRAMES},
        {"videoinput",     required_argument, NULL, ArgID_VIDEOINPUT},
        {"bitrate",        required_argument, NULL, ArgID_BITRATE},
        {"accel",          no_argument,       NULL, ArgID_ACCEL},
        {"capture_dalloc", no_argument,       NULL, ArgID_CAPTURE_DALLOC},
        {"xdm",            required_argument, NULL, ArgID_XDM},
        {"help",           no_argument,       NULL, ArgID_HELP        },
        {"output_file",    required_argument, NULL, ArgID_OUTPUT_FILE  },
        {0, 0, 0, 0}
    };
    Int     codec = FALSE;
    Int     outFile = FALSE;
    Int     index;
    Int     argID;

    /* Setting default values */
    strncpy(argsp->engineName, DEFAULT_ENGINE_NAME, MAX_ENGINE_NAME_SIZE);

    argsp->bitRate = -1;
    for (;;) {
        argID = getopt_long(argc, argv, shortOptions, longOptions, &index);

        if (argID == -1) {
            break;
        }

        switch (argID) {

            case ArgID_CODEC:
            case 'c':
                strncpy(argsp->codecName, optarg, MAX_CODEC_NAME_SIZE);
                codec = TRUE;
                break;

            case ArgID_ENGINE:
            case 'e':
                strncpy(argsp->engineName, optarg, MAX_ENGINE_NAME_SIZE);
                break;

            case ArgID_HELP:
            case 'h':
                usage();
                exit(EXIT_SUCCESS);

            case ArgID_OUTPUT_FILE:
            case 'o':
                strncpy(argsp->outFile, optarg, MAX_FILE_NAME_SIZE);
                outFile = TRUE;
                break;
                
            case ArgID_NUMFRAMES:
            case 'n':
                argsp->numFrames = atoi(optarg);
                break;
                
            case ArgID_VIDEOINPUT:
                switch (atoi(optarg)) {
                    case 1:
                        argsp->videoInput = Capture_Input_COMPOSITE;
                        break;
                    case 2:
                        argsp->videoInput = Capture_Input_COMPONENT;
                        break;
                    case 3:
                        argsp->videoInput = Capture_Input_SVIDEO;
                        break;
                    default:
                        fprintf(stderr, "Unknown video input %d\n",
                                atoi(optarg));
                        usage();
                        exit(EXIT_FAILURE);
                }
                break;

            case ArgID_BITRATE:
        case 'b':
                argsp->bitRate = atoi(optarg);
                break;

            case ArgID_ACCEL:
                argsp->accel = TRUE;
                break;

            case ArgID_CAPTURE_DALLOC:
                argsp->captureAlloc = TRUE;
                break;

            case ArgID_XDM:
                if (strcmp(optarg, "0.9") == 0) {
                    argsp->vXdm = Xdm0;
                }
                else if (strcmp(optarg, "1.0") == 0) {
                    argsp->vXdm = Xdm1;
                }
                else {
                    fprintf(stderr, "Only XDM 0.9 and 1.0 supported\n\n");
                    usage();
                    exit(EXIT_FAILURE);
                }
                break;

            case ArgID_BENCHMARK:
                argsp->benchmark = TRUE;
                break;
                
            default:
                usage();
                exit(EXIT_FAILURE);
        }
    }

    if (optind < argc) {
        usage();
        exit(EXIT_FAILURE);
    }
    if (!codec || !outFile) {
        usage();
        exit(EXIT_FAILURE);
    }
}



/******************************************************************************
 * main
 ******************************************************************************/
Int main(Int argc, Char *argv[])
{
    Args args = DEFAULT_ARGS;
    
    /* Parse the arguments given to the app */
    parseArgs(argc, argv, &args);    

    appMain(&args);
    
    return 0;
}
