/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/ce/Engine.h>
#include <ti/sdo/ce/CERuntime.h>

#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Time.h>
#include <ti/sdo/dmai/BufTab.h>
#include <ti/sdo/dmai/Loader.h>
#include <ti/sdo/dmai/Display.h>
#include <ti/sdo/dmai/BufferGfx.h>
#include <ti/sdo/dmai/Framecopy.h>
#include <ti/sdo/dmai/ColorSpace.h>
#include <ti/sdo/dmai/ce/Vdec.h>

#include "appMain.h"

#define NUM_BUFS                2
#define NUM_DISPLAY_BUFS        3

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    Display_Attrs           dAttrs    = Display_Attrs_DM6446_DM355_VID_DEFAULT;
    VIDDEC_Params           params      = Vdec_Params_DEFAULT;
    VIDDEC_DynamicParams    dynParams   = Vdec_DynamicParams_DEFAULT;
    Framecopy_Attrs         fcAttrs     = Framecopy_Attrs_DEFAULT;
    Loader_Attrs            lAttrs      = Loader_Attrs_DEFAULT;
    BufferGfx_Attrs         gfxAttrs    = BufferGfx_Attrs_DEFAULT;
    Time_Attrs              tAttrs      = Time_Attrs_DEFAULT;
    Framecopy_Handle        hFc         = NULL;
    Vdec_Handle             hVd         = NULL;
    Display_Handle          hDisplay    = NULL;
    Loader_Handle           hLoader     = NULL;
    Engine_Handle           hEngine     = NULL;
    BufTab_Handle           hDisBufTab  = NULL;
    BufTab_Handle           hBufTab     = NULL;
    Buffer_Handle           hDstBuf     = NULL;
    Time_Handle             hTime       = NULL;
    Int                     ret         = Dmai_EOK;
    Int                     numFrame    = 0;
    BufferGfx_Dimensions    dim;
    Buffer_Handle           hInBuf, hDispBuf, hOutBuf;
    Int                     bufSize;
    UInt32                  time;

    if (args->benchmark) {
        hTime = Time_create(&tAttrs);

        if (hTime == NULL) {
            printf("Failed to create Time object\n");
            goto cleanup;
        }
    }

    /* Initialize the codec engine run time */
    CERuntime_init();

    /* Initialize DMAI */
    Dmai_init();
    
    dAttrs.videoStd = args->videoStd;
    dAttrs.numBufs = NUM_DISPLAY_BUFS;

    if (args->displayAlloc) {
        /* Create the video display */
        hDisplay = Display_create(NULL, &dAttrs);
    }
    else {
        if (BufferGfx_calcDimensions(args->videoStd,
                                  ColorSpace_UYVY, &gfxAttrs.dim) < 0) {
            printf("Failed to calculate buffer dimensions\n");
            goto cleanup;
        }

        bufSize = BufferGfx_calcSize(args->videoStd, ColorSpace_UYVY);

        if (bufSize < 0) {
            printf("Failed to calculate buffer size\n");
            goto cleanup;
        }

        hDisBufTab = BufTab_create(NUM_DISPLAY_BUFS, bufSize,
                                   BufferGfx_getBufferAttrs(&gfxAttrs));

        if (hDisBufTab == NULL) {
            printf("Failed to allocate contiguous buffers\n");
            goto cleanup;
        }

        /* Create the video display */
        hDisplay = Display_create(hDisBufTab, &dAttrs);
    }

    if (hDisplay == NULL) {
        printf("Failed to open display device\n");
        goto cleanup;
    }

    if (BufferGfx_calcDimensions(args->videoStd,
                              ColorSpace_UYVY, &gfxAttrs.dim) < 0) {
        printf("Failed to calculate buffer dimensions\n");
        goto cleanup;
    }

    bufSize = BufferGfx_calcSize(args->videoStd, ColorSpace_UYVY);

    if (bufSize < 0) {
        printf("Failed to calculate buffer size\n");
        goto cleanup;
    }

    gfxAttrs.colorSpace = ColorSpace_UYVY;
    
    hBufTab = BufTab_create(NUM_BUFS, bufSize,
                            BufferGfx_getBufferAttrs(&gfxAttrs));

    if (hBufTab == NULL) {
        printf("Failed to allocate contiguous buffers\n");
        goto cleanup;
    }

    /* Open the codec engine */
    hEngine = Engine_open(args->engineName, NULL, NULL);

    if (hEngine == NULL) {
        printf("Failed to open codec engine %s\n", args->engineName);
        goto cleanup;
    }

    /* Create the XDM 0.9 based video decoder */
    hVd = Vdec_create(hEngine, args->codecName, &params, &dynParams);

    if (hVd == NULL) {
        printf("Failed to create video decoder: %s\n", args->codecName);
        goto cleanup;
    }

    /* Set output buffer table */
    Vdec_setBufTab(hVd, hBufTab);
    
    fcAttrs.accel = args->accel;

    hFc = Framecopy_create(&fcAttrs);

    if (hFc == NULL) {
        printf("Failed to configure frame copy job\n");
        goto cleanup;
    }

printf("source: %d\n", BufferGfx_getColorSpace(BufTab_getBuf(hBufTab, 0)));
printf("dest: %d\n", BufferGfx_getColorSpace(BufTab_getBuf(Display_getBufTab(hDisplay), 0)));

    if (Framecopy_config(hFc, BufTab_getBuf(hBufTab, 0),
                         BufTab_getBuf(Display_getBufTab(hDisplay), 0)) < 0) {
        printf("Failed to configure frame copy job\n");
        goto cleanup;
    }

    printf("Frame copy job created and configured\n");

    lAttrs.readSize = Vdec_getInBufSize(hVd);
    lAttrs.readBufSize = lAttrs.readSize * 2; //2097152;

    /* Create the file loader */
    hLoader = Loader_create(args->inFile, &lAttrs);

    if (hLoader == NULL) {
        printf("Failed to create loader\n");
        goto cleanup;
    }

    /* Prime the file loader */
    Loader_prime(hLoader, &hInBuf);

    printf("Loader created and primed, entering main loop..\n");

    while (numFrame++ < args->numFrames) {
        if (ret != Dmai_EPRIME &&
            ret != Dmai_EFIRSTFIELD &&
            ret != Dmai_EBITERROR) {

            /* Get a buffer from the display driver */
            if (Display_get(hDisplay, &hDispBuf) < 0) {
                printf("Failed to get display buffer\n");
                goto cleanup;
            }
        }

        if (ret != Dmai_EFIRSTFIELD && ret != Dmai_EBITERROR) {
            hDstBuf = BufTab_getFreeBuf(hBufTab);

            if (hDstBuf == NULL) {
                printf("Failed to get a free contiguous buffer from BufTab\n");
                goto cleanup;
            }
        }

        if (args->benchmark) {
            if (Time_reset(hTime) < 0) {
                printf("Failed to reset timer\n");
                goto cleanup;
            }
        }

        /* Decode the video buffer */
        ret = Vdec_process(hVd, hInBuf, hDstBuf);

        if (ret < 0) {
            printf("Failed to decode video buffer\n");
            goto cleanup;
        }

        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }

            printf("%d: Decode: %uus ", numFrame, (Uns) time);
        }

        /* Do we have a valid frame to be displayed? */
        if (ret != Dmai_EPRIME &&
            ret != Dmai_EFIRSTFIELD &&
            ret != Dmai_EBITERROR) {

            /* Get display buffer from codec */
            hOutBuf = Vdec_getDisplayBuf(hVd);

            BufferGfx_getDimensions(hOutBuf, &dim);

#if 0 // Use dimensions instead
            x = (dim.width - dim.width) / 2 * 2; /*16 bpp*/
            y = (dim.height - dim.height) / 2 & ~1;
            offset = (y * dim.lineLength + x) & ~0x1f;
#endif

            if (Framecopy_execute(hFc, hOutBuf, hDispBuf) < 0) {
                printf("Failed to execute frame copy job\n");
                goto cleanup;
            }

            if (args->benchmark) {
                if (Time_delta(hTime, &time) < 0) {
                    printf("Failed to get timer delta\n");
                    goto cleanup;
                }

                printf("CCV / Framecopy: %uus ", (Uns) time);
            }

            BufTab_freeBuf(hOutBuf);

            /* Display the decoded buffer */
            if (Display_put(hDisplay, hDispBuf) < 0) {
                printf("Failed to put display buffer\n");
                goto cleanup;
            }

        }

        /* Load a new frame from the file system */
        Loader_getFrame(hLoader, hInBuf);

        if (Buffer_getUserPtr(hInBuf) == NULL) {
            printf("Loader returned null, clip finished\n");
            break;
        }

        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }

            printf("Loader: %uus ", (Uns) time);
        }

        if (args->benchmark) {
            if (Time_total(hTime, &time) < 0) {
                printf("Failed to get timer total\n");
                goto cleanup;
            }

            printf("Total: %uus\n", (Uns) time);
        }
    }

cleanup:
    /* Clean up the application */
    if (hLoader) {
        Loader_delete(hLoader);
    }

    if (hVd) {
        Vdec_delete(hVd);
    }

    if (hFc) {
        Framecopy_delete(hFc);
    }

    if (hDisplay) {
        Display_delete(hDisplay);
    }

    if (hDisBufTab) {
        BufTab_delete(hDisBufTab);
    }

    if (hBufTab) {
        BufTab_delete(hBufTab);
    }

    if (hEngine) {
        Engine_close(hEngine);
    }

    if (hTime) {
        Time_delete(hTime);
    }

    return;
}
