/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/ce/Engine.h>
#include <ti/sdo/ce/CERuntime.h>

#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Ccv.h>
#include <ti/sdo/dmai/Cpu.h>
#include <ti/sdo/dmai/Time.h>
#include <ti/sdo/dmai/BufTab.h>
#include <ti/sdo/dmai/Loader.h>
#include <ti/sdo/dmai/Display.h>
#include <ti/sdo/dmai/VideoStd.h>
#include <ti/sdo/dmai/ce/Vdec2.h>
#include <ti/sdo/dmai/Framecopy.h>
#include <ti/sdo/dmai/ColorSpace.h>
#include <ti/sdo/dmai/BufferGfx.h>

#include "appMain.h"

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    VIDDEC2_Params          params      = Vdec2_Params_DEFAULT;
    VIDDEC2_DynamicParams   dynParams   = Vdec2_DynamicParams_DEFAULT;
    Loader_Attrs            lAttrs      = Loader_Attrs_DEFAULT;
    BufferGfx_Attrs         gfxAttrs    = BufferGfx_Attrs_DEFAULT;
    Ccv_Attrs               ccvAttrs    = Ccv_Attrs_DEFAULT;
    Framecopy_Attrs         fcAttrs     = Framecopy_Attrs_DEFAULT;
    Time_Attrs              tAttrs      = Time_Attrs_DEFAULT;
    Vdec2_Handle            hVd2        = NULL;
    Ccv_Handle              hCcv        = NULL;
    Framecopy_Handle        hFc         = NULL;
    Display_Handle          hDisplay    = NULL;
    Loader_Handle           hLoader     = NULL;
    Engine_Handle           hEngine     = NULL;
    BufTab_Handle           hDisBufTab  = NULL;
    BufTab_Handle           hBufTab     = NULL;
    Time_Handle             hTime       = NULL;
    Buffer_Handle           hDstBuf     = NULL;
    Int                     ret         = Dmai_EOK;
    Int                     numFrame    = 0;
    Int                     numBufs     = 3;
    Int                     numDispBufs = 3;
    Buffer_Handle           hDispBuf, hOutBuf, hFreeBuf, hInBuf;
    Display_Attrs           dAttrs;
    Int                     bufSize;
    Cpu_Device              device;
    UInt32                  time;

    /* Initialize DMAI */
    Dmai_init();
    
    if (Cpu_getDevice(NULL, &device) < 0) {
        printf("Failed to determine target board\n");
        goto cleanup;
    }

    if (device == Cpu_Device_DM6467) {
        dAttrs = Display_Attrs_DM6467_VID_DEFAULT;
    }
    else {
        dAttrs = Display_Attrs_DM6446_DM355_VID_DEFAULT;
    }

    if (args->benchmark) {
        hTime = Time_create(&tAttrs);

        if (hTime == NULL) {
            printf("Failed to create Time object\n");
            goto cleanup;
        }
    }

    /* Initialize the codec engine run time */
    CERuntime_init();

    dAttrs.videoStd = args->videoStd;
    dAttrs.numBufs  = numDispBufs;

    if (args->displayAlloc) {
        /* Create the video display */
        hDisplay = Display_create(NULL, &dAttrs);
    }
    else {
        if (BufferGfx_calcDimensions(args->videoStd,
                                      ColorSpace_YUV422PSEMI,
                                      &gfxAttrs.dim) < 0) {
            printf("Failed to calculate dimensions for display buffers\n");
            goto cleanup;
        }

        bufSize = BufferGfx_calcSize(args->videoStd, ColorSpace_YUV422PSEMI);

        if (bufSize < 0) {
            printf("Failed to calculate size for display buffers\n");
            goto cleanup;
        }

        hDisBufTab = BufTab_create(numDispBufs, bufSize,
                                   (Buffer_Attrs *) &gfxAttrs);

        if (hDisBufTab == NULL) {
            printf("Failed to allocate contiguous buffers\n");
            goto cleanup;
        }

        /* Create the video display */
        hDisplay = Display_create(hDisBufTab, &dAttrs);
    }

    if (hDisplay == NULL) {
        printf("Failed to open display device\n");
        goto cleanup;
    }

    /* Open the codec engine */
    hEngine = Engine_open(args->engineName, NULL, NULL);

    if (hEngine == NULL) {
        printf("Failed to open codec engine %s\n", args->engineName);
        goto cleanup;
    }

    if (VideoStd_getResolution(args->videoStd, &params.maxWidth,
                               &params.maxHeight) < 0) {
        printf("Invalid video standard (%d)\n", args->videoStd);
        goto cleanup;
    }

    if (device != Cpu_Device_DM6467) {
       params.forceChromaFormat = XDM_YUV_422ILE;
    }

    /* TODO! Set here or use defaults? */
    params.maxFrameRate        = 30000;
    params.maxBitRate          = 1000000;
    params.dataEndianness      = XDM_BYTE;

    dynParams.decodeHeader     = 0;
    dynParams.displayWidth     = 0;
    dynParams.frameSkipMode    = 0;
    dynParams.frameOrder       = 0;
    dynParams.newFrameFlag     = 0;
    dynParams.mbDataFlag       = 0;

    /* Create the XDM 1.2 based video decoder */
    hVd2 = Vdec2_create(hEngine, args->codecName, &params, &dynParams);

    if (hVd2 == NULL) {
        printf("Failed to create video decoder: %s\n", args->codecName);
        goto cleanup;
    }

    /*
     * Override the calculated buffer size with the codec estimation since
     * some codecs pad the buffer sizes needed.
     */
    bufSize = Vdec2_getOutBufSize(hVd2);

    gfxAttrs.colorSpace = device == Cpu_Device_DM6467 ?
                    ColorSpace_YUV420PSEMI : ColorSpace_UYVY;

    if (BufferGfx_calcDimensions(args->videoStd, gfxAttrs.colorSpace,
                                  &gfxAttrs.dim) < 0) {
        printf("Failed to calculate dimensions for display buffers\n");
        goto cleanup;
    }

    hBufTab = BufTab_create(numBufs, bufSize, (Buffer_Attrs *) &gfxAttrs);

    if (hBufTab == NULL) {
        printf("Failed to allocate contiguous buffers\n");
        goto cleanup;
    }

    Vdec2_setBufTab(hVd2, hBufTab);

    if (device == Cpu_Device_DM6467) {
        ccvAttrs.accel = args->accel;

        hCcv = Ccv_create(&ccvAttrs);

        if (hCcv == NULL) {
            printf("Failed to create color conversion job\n");
            goto cleanup;
        }

        if (Ccv_config(hCcv, BufTab_getBuf(hBufTab, 0),
                       BufTab_getBuf(Display_getBufTab(hDisplay), 0)) < 0) {
            printf("Failed to configure color conversion job\n");
            goto cleanup;
        }
    }
    else {
        fcAttrs.accel = args->accel;

        hFc = Framecopy_create(&fcAttrs);

        if (hFc == NULL) {
            printf("Failed to configure frame copy job\n");
            goto cleanup;
        }

        if (Framecopy_config(hFc, BufTab_getBuf(hBufTab, 0),
                             BufTab_getBuf(Display_getBufTab(hDisplay), 0)
                             ) < 0) {
            printf("Failed to configure frame copy job\n");
            goto cleanup;
        }
    }

    printf("Color conversion job created and configured\n");

    /* Create the file loader */
    lAttrs.readSize = device == Cpu_Device_DM6467 ?
                                    Vdec2_getInBufSize(hVd2) : 260000;
    lAttrs.readBufSize = (device == Cpu_Device_DM6467) ?
                                    4147200 : 2097152;
    hLoader = Loader_create(args->inFile, &lAttrs);

    if (hLoader == NULL) {
        printf("Failed to create loader\n");
        goto cleanup;
    }

    /* Prime the file loader */
    Loader_prime(hLoader, &hInBuf);

    printf("Loader created and primed, entering main loop..\n");

    while (numFrame++ < args->numFrames) {
        if (ret != Dmai_EFIRSTFIELD && ret != Dmai_EBITERROR) {

            Buffer_Handle hNextDstBuf = BufTab_getFreeBuf(hBufTab);

            if (hNextDstBuf == NULL && device != Cpu_Device_DM355) {
                printf("Failed to get a free contiguous buffer from BufTab\n");
                goto cleanup;
            }

            if (hNextDstBuf) {
                hDstBuf = hNextDstBuf;
            }
        }

        if (args->benchmark) {
            if (Time_reset(hTime) < 0) {
                printf("Failed to reset timer\n");
                goto cleanup;
            }
        }

        /* Decode the video buffer */
        ret = Vdec2_process(hVd2, hInBuf, hDstBuf);

        if (ret < 0) {
            printf("Failed to decode video buffer\n");
            goto cleanup;
        }

        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }

            printf("%d: Decode: %uus ", numFrame, (Uns) time);
        }

        /* Display frames */
        hOutBuf = Vdec2_getDisplayBuf(hVd2);
        while (hOutBuf) {
            /* Get a buffer from the display driver */
            if (Display_get(hDisplay, &hDispBuf) < 0) {
                printf("Failed to get display buffer\n");
                goto cleanup;
            }

            if (device == Cpu_Device_DM6467) {
                if (Ccv_execute(hCcv, hOutBuf, hDispBuf) < 0) {
                    printf("Failed to execute color conversion job\n");
                    goto cleanup;
                }
            }
            else {
                if (Framecopy_execute(hFc, hOutBuf, hDispBuf) < 0) {
                    printf("Failed to execute frame copy job\n");
                    goto cleanup;
                }
            }

            if (args->benchmark) {
                if (Time_delta(hTime, &time) < 0) {
                    printf("Failed to get timer delta\n");
                    goto cleanup;
                }

                printf("CCV: %uus ", (Uns) time);
            }

            /* Display the decoded buffer */
            if (Display_put(hDisplay, hDispBuf) < 0) {
                printf("Failed to put display buffer\n");
                goto cleanup;
            }

            hOutBuf = Vdec2_getDisplayBuf(hVd2);
        }

        /* Free up released frames */
        hFreeBuf = Vdec2_getFreeBuf(hVd2);
        while (hFreeBuf) {
            BufTab_freeBuf(hFreeBuf);
            hFreeBuf = Vdec2_getFreeBuf(hVd2);
        }

        /* Load a new encoded frame from the file system */
        Loader_getFrame(hLoader, hInBuf);

        if (Buffer_getUserPtr(hInBuf) == NULL) {
            printf("Loader returned null, clip finished\n");
            break;
        }

        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }

            printf("Loader: %uus ", (Uns) time);
        }

        if (args->benchmark) {
            if (Time_total(hTime, &time) < 0) {
                printf("Failed to get timer total\n");
                goto cleanup;
            }

            printf("Total: %uus\n", (Uns) time);
        }
    }

cleanup:
    /* Clean up the application */
    if (hLoader) {
        Loader_delete(hLoader);
    }

    if (hVd2) {
        Vdec2_delete(hVd2);
    }

    if (hCcv) {
        Ccv_delete(hCcv);
    }

    if (hDisplay) {
        Display_delete(hDisplay);
    }

    if (hDisBufTab) {
        BufTab_delete(hDisBufTab);
    }

    if (hBufTab) {
        BufTab_delete(hBufTab);
    }

    if (hEngine) {
        Engine_close(hEngine);
    }

    if (hTime) {
        Time_delete(hTime);
    }

    return;
}
