/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>

#include <xdc/std.h>

#include "../appMain.h"

#define DEFAULT_ENGINE_NAME     "decode"

/*
 * Argument IDs for long options. They must not conflict with ASCII values,
 * so start them at 256.
 */
typedef enum
{
   ArgID_BENCHMARK = 256,
   ArgID_CODEC,
   ArgID_ENGINE,
   ArgID_HELP,
   ArgID_INPUT_FILE,
   ArgID_NUMFRAMES,
   ArgID_RESOLUTION,
   ArgID_ACCEL,
   ArgID_DISPLAY_DALLOC,
   ArgID_NUMARGS
} ArgID;


#define DEFAULT_ARGS { 30 * 30, VideoStd_D1_NTSC, FALSE, FALSE, FALSE }

/******************************************************************************
 * usage
 ******************************************************************************/
static Void usage(void)
{
    fprintf(stderr, "Usage: video_decode2_<platform> [options]\n\n"
        "Options:\n"
        "   | --benchmark      Benchmark values are printed.\n"
        "-c | --codec          Name of codec to use\n"
        "-e | --engine         Codec engine containing specified codec\n"
        "-n | --numframes      Number of frames to encode [Default: 1000].\n"
        "-r | --resolution     Resolution to use for display.\n"
        "   | --accel          Use hardware acceleration if available for\n"
        "                      output color space conversion or frame copy.\n"
        "   | --display_dalloc Have the display driver allocate own buffers.\n"
        "-h | --help           Print usage information (this message)\n"
        "-i | --input_file     Name of input file\n"
        "At a minimum the codec name and the file name *must* be given.\n\n"
        "Resolutions available:\n"
        "\t1\tD1 @ 30 fps (NTSC) [Default]\n"
        "\t2\tD1 @ 25 fps (PAL)\n"
        "\t3\t720P @ 60fps\n"
        "\t4\t720P @ 50fps\n"
        "\t5\t1080i @ 30fps\n"
        "\t6\t1080i @ 25fps\n\n");
}

/******************************************************************************
 * parseArgs
 ******************************************************************************/
static Void parseArgs(Int argc, Char *argv[], Args *argsp)
{  
    const char shortOptions[] = "c:e:n:r:hi:";
    const struct option longOptions[] = {
        {"benchmark",      no_argument,       NULL, ArgID_BENCHMARK   },
        {"codec",          required_argument, NULL, ArgID_CODEC       },
        {"engine",         required_argument, NULL, ArgID_ENGINE      },
        {"numframes",      required_argument, NULL, ArgID_NUMFRAMES   },
        {"resoluton",      required_argument, NULL, ArgID_RESOLUTION  },
        {"accel",          no_argument,       NULL, ArgID_ACCEL       },
        {"display_dalloc", no_argument,       NULL, ArgID_DISPLAY_DALLOC},
        {"help",           no_argument,       NULL, ArgID_HELP        },
        {"input_file",     required_argument, NULL, ArgID_INPUT_FILE  },
        {0, 0, 0, 0}
    };
    Int     codec = FALSE;
    Int     file = FALSE;
    Int     index;
    Int     argID;

    /* Setting default values */
    strncpy(argsp->engineName, DEFAULT_ENGINE_NAME, MAX_ENGINE_NAME_SIZE);

    for (;;) {
        argID = getopt_long(argc, argv, shortOptions, longOptions, &index);

        if (argID == -1) {
            break;
        }

        switch (argID) {

            case ArgID_CODEC:
            case 'c':
                strncpy(argsp->codecName, optarg, MAX_CODEC_NAME_SIZE);
                codec = TRUE;
                break;

            case ArgID_ENGINE:
            case 'e':
                strncpy(argsp->engineName, optarg, MAX_ENGINE_NAME_SIZE);
                break;

            case ArgID_HELP:
            case 'h':
                usage();
                exit(EXIT_SUCCESS);

            case ArgID_RESOLUTION:
        case 'r':
                switch (atoi(optarg)) {
                    case 1:
                        argsp->videoStd = VideoStd_D1_NTSC;
                        break;
                    case 2:
                        argsp->videoStd = VideoStd_D1_PAL;
                        break;
                    case 3:
                        argsp->videoStd = VideoStd_720P_60;
                        break;
                    case 4:
                        argsp->videoStd = VideoStd_720P_50;
                        break;
                    case 5:
                        argsp->videoStd = VideoStd_1080I_30;
                        break;
                    case 6:
                        argsp->videoStd = VideoStd_1080I_25;
                        break;
                    default:
                        fprintf(stderr, "Unknown display resolution\n");
                        usage();
                        exit(EXIT_FAILURE);
                }
                break;

            case ArgID_INPUT_FILE:
            case 'i':
                strncpy(argsp->inFile, optarg, MAX_FILE_NAME_SIZE);
                file = TRUE;
                break;
                
            case ArgID_NUMFRAMES:
            case 'n':
                argsp->numFrames = atoi(optarg);
                break;
                

            case ArgID_ACCEL:
                argsp->accel = TRUE;
                break;

            case ArgID_DISPLAY_DALLOC:
                argsp->displayAlloc = TRUE;
                break;

            case ArgID_BENCHMARK:
                argsp->benchmark = TRUE;
                break;
                
            default:
                usage();
                exit(EXIT_FAILURE);
        }
    }

    if (optind < argc) {
        usage();
        exit(EXIT_FAILURE);
    }
    if (!codec || !file) {
        usage();
        exit(EXIT_FAILURE);
    }
}



/******************************************************************************
 * main
 ******************************************************************************/
Int main(Int argc, Char *argv[])
{
    Args args = DEFAULT_ARGS;
    
    /* Parse the arguments given to the app */
    parseArgs(argc, argv, &args);    

    appMain(&args);
    
    return 0;
}
